@@ -18,7 +18,7 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: util.c,v 1.131 2004/11/05 10:01:32 kmaraas Exp $
+    $Id: util.c,v 1.133 2005/02/23 09:13:06 tml Exp $
 
 ***************************************************************************/
 #include <assert.h>
@@ -31,9 +31,14 @@
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
+
 #include "rename.h"
 #include "util.h"
 
+#ifdef G_OS_WIN32
+#include <fcntl.h>
+#endif
+
 IDL_EXPORT const char *IDL_tree_type_names[] = {
 	"IDLN_NONE",
 	"IDLN_ANY",
@@ -218,18 +223,22 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	extern FILE *__IDL_in;
 	FILE *input;
 	char *cmd;
-	size_t cmd_len;
 #ifdef HAVE_CPP_PIPE_STDIN
 	char *fmt = CPP_PROGRAM " " CPP_NOSTDINC " - %s%s %s < \"%s\" %s";
-	char *wd = "", *dirend;
+	char *wd;
 #else
 	char *fmt = CPP_PROGRAM " " CPP_NOSTDINC " -I- -I%s %s \"%s\" %s";
 	char *s, *tmpfilename;
 	char cwd[2048];
 	gchar *linkto;
 #endif
+
+#ifndef G_OS_WIN32
 	char *cpperrs = (parse_flags & IDLF_SHOW_CPP_ERRORS) 
 	  ? "" : "2>/dev/null";
+#else
+	char *cpperrs = "";
+#endif
 	GSList *slist;
 	int rv;
 
@@ -253,28 +262,12 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 #endif
 
 #ifdef HAVE_CPP_PIPE_STDIN
-	if ((dirend = strrchr (filename, '/'))) {
-		int len = dirend - filename + 1;
-		wd = g_malloc (len);
-		strncpy (wd, filename, len - 1);
-		wd[len - 1] = 0;
-	}
+	wd = g_path_get_dirname (filename);
 
-	cmd_len = (strlen (filename) + (*wd ? 2 : 0) + strlen (wd) +
-		  (cpp_args ? strlen (cpp_args) : 0) +
-		  strlen(cpperrs) +
-		  strlen (fmt) - 8 + 1);
-	cmd = g_malloc (cmd_len);
-	if (!cmd) {
-		errno = ENOMEM;
-		return -1;
-	}
+	cmd = g_strdup_printf (fmt, "-I", wd, cpp_args ? cpp_args : "",
+			       filename, cpperrs);
 
-	g_snprintf (cmd, cmd_len, fmt, *wd ? "-I" : "", wd,
-		    cpp_args ? cpp_args : "", filename, cpperrs);
-
-	if (dirend)
-		g_free (wd);
+	g_free (wd);
 #else
 	s = tmpnam (NULL);
 	if (s == NULL)
@@ -283,58 +276,50 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	if (!getcwd (cwd, sizeof (cwd)))
 		return -1;
 
-	if (*filename == '/') {
+	if (g_path_is_absolute (filename)) {
 		linkto = g_strdup (filename);
 	} else {
-		linkto = g_malloc (strlen (cwd) + strlen (filename) + 2);
-		if (!linkto) {
-			errno = ENOMEM;
-			return -1;
-		}
-		strcpy (linkto, cwd);
-		strcat (linkto, "/");
-		strcat (linkto, filename);
+		linkto = g_strconcat (cwd, "/", filename, NULL);
 	}
 
-	tmpfilename = g_malloc (strlen (s) + 3);
-	if (!tmpfilename) {
-		g_free (linkto);
-		errno = ENOMEM;
-		return -1;
-	}
-	strcpy (tmpfilename, s);
-	strcat (tmpfilename, ".c");
+	tmpfilename = g_strconcat (s, ".c", NULL);
 #ifdef HAVE_SYMLINK
 	if (symlink (linkto, tmpfilename) < 0) {
 		g_free (linkto);
 		g_free (tmpfilename);
 		return -1;
 	}
+#else
+#error Must have symlink
 #endif
 	g_free (linkto);
 
-	cmd_len = (strlen (tmpfilename) + strlen (cwd) +
-		   (cpp_args ? strlen (cpp_args) : 0) +
-		   strlen(cpperrs) +
-		   strlen (fmt) - 6 + 1);
-	cmd = g_malloc (cmd_len);
-	if (!cmd) {
-		g_free (tmpfilename);
-		errno = ENOMEM;
-		return -1;
-	}
-
-	g_snprintf (cmd, cmd_len, fmt,
-		    cwd, cpp_args ? cpp_args : "", tmpfilename, cpperrs);
+	cmd = g_strdup_printf (fmt, cwd, cpp_args ? cpp_args : "",
+			       tmpfilename, cpperrs);
 #endif
 
        /* Many versions of cpp do evil translating internal
         * strings, producing bogus output, so clobber LC_ALL */
-       putenv ("LC_ALL=C");
+	putenv ("LC_ALL=C");
 
 #ifdef HAVE_POPEN
-	input = popen (cmd, "r");
+#ifdef G_OS_WIN32
+	if (!(parse_flags & IDLF_SHOW_CPP_ERRORS)) {
+		int save_stderr = dup (2);
+		int null = open ("NUL:", O_WRONLY);
+		dup2 (null, 2);
+		input = popen (cmd, "r");
+		close (2);
+		close (null);
+		dup2 (save_stderr, 2);
+		close (save_stderr);
+	} else
+		input = popen (cmd, "r");
 #else
+	input = popen (cmd, "r");
+#endif
+#else
+	/* Surely this doesn't make any sense? */	
 	input = fopen (cmd, "r");
 #endif
 	g_free (cmd);
